define(function (require) {
  var HAResources = require('ha-resources');
  var MultiRegion = require('multi-region');

  var App = new Backbone.Marionette.Application();

  App.name = 'MCC Support';
  App.version = '1.0.0';
  App.showWarningInterval = 180000;    //ms
  App.sleepCheckInterval  = 10 * 2000; //ms
  App.sleepCheckThreshold = 30  * 2000; //ms

  App.Resources = new HAResources(App);

  App.addRegions({
    contentRegion: '#content',
    headerRegion: 'header',
    bodyRegion: 'body',
    popupRegion: new MultiRegion({el: '#popup-region'}),
    footerRegion: 'footer'

  });
  App.focusPopup = function () {
    // specialized focusing
    var focus = function () {
      var $focus = $('.focus-second');
      if ($focus.length > 0) {
        $focus.focus().click();
      }

    };
    setTimeout(focus, 150);
  };
  App.goToTop = function () {
    window.scrollTo(0, 0);
  };
  App.focusTop = function () {
    // specialized focusing
    var focus = function () {
      var $focus = $('.focus-first');
      if ($focus.length === 0) {
        $focus = $('#header > h1');
      }
      $focus.focus();
    };
    setTimeout(focus, 150);
  };
  App.getModel = function () {
    return window.sessionStorage.getItem('model');
  };
  App.setModel = function (model) {
    window.sessionStorage.setItem('model', model);
  };

  //<----------------------------------------- Session ----------------------------------------------->

  App.login = function () {
    App.Resources.login();
  };

  App.startSessionTimer = function () {

    var timeLeft = App.Resources.getTimeToExpireInMilliSeconds();
    if (App.warningTimer) {
      clearTimeout(App.warningTimer);
    }

    if (App.endedTimer) {
      clearTimeout(App.endedTimer);
    }

    if (timeLeft > 0) {
      App.warningTimer = window.setTimeout(function () {
        App.vent.trigger('session:warning');
      }, timeLeft - App.showWarningInterval);

      App.endedTimer = window.setTimeout(function () {
        App.vent.trigger('session:ended');
      }, timeLeft);
    }

  };
  App.resetSessionTimer = function () {
    App.Resources.resetSessionTimer();
    App.startSessionTimer();
  };

  App.cleanUpSession = function () {
    if (App.warningTimer) {
      clearTimeout(App.warningTimer);
    }

    if (App.endedTimer) {
      clearTimeout(App.endedTimer);
    }

    if (App.sessionSleepThread) {
      clearTimeout(App.sessionSleepThread);
    }

    App.Resources.cleanUpSession();
  };
  App.noAjax = false;
  //<----------------------------------------- Ajax    ----------------------------------------------->
  $(document).ajaxStart(function () {
      $('.ajax-loading').show();
  }).ajaxStop(function () {
    $('.ajax-loading').hide();
  }).ajaxError(function (event, jqxhr, settings, thrownError) {
    if (thrownError === 'timeout') {
      App.router.popups.showAjaxTimeout();
    }
  });

  //<----------------------------------------- Global Events ------------------------------------------>
  function isiPhone() {
    return (
      //Detect iPhone
      //var isiPad = navigator.userAgent.match(/iPad/i) != null;
      (navigator.platform.indexOf('iPhone') !== -1) ||
      //Detect iPod
      (navigator.platform.indexOf('iPad') !== -1)
      );
  }
  if (isiPhone()) {
    var $body = $('.container-full');
    var $footer = $('footer');
    var $header = $('header');

    $(document)
      .on('focus', 'input', function (e) {
        $body.addClass('fixfixed');
        $footer.removeClass('footer-fixed');
        $footer.addClass('footer');
        $header.removeClass('header-fixed');
        $header.addClass('mobile-header');
      })
      .on('blur', 'input', function (e) {
        $body.removeClass('fixfixed');
        $footer.addClass('footer-fixed');
        $footer.removeClass('footer');
        $header.addClass('header-fixed');
        $header.removeClass('mobile-header');
      });
  }

  App.vent.on('route:startup', function () {
    Backbone.history.start();
  });
  App.contentRegion.listenTo(App.vent, 'menu:open', function () {
    this.$el.attr('aria-hidden', true);
  });
  App.contentRegion.listenTo(App.vent, 'menu:close', function () {
    this.$el.removeAttr('aria-hidden');
  });
  App.contentRegion.on('show', function () {
    App.focusTop();
  });

  //<------------------------------------------ Init ------------------------------------------------------>


  if (App.Resources.isLoggedIn()) {
    //session timeout timer
    App.startSessionTimer();
    $(document).on('ajaxSuccess', function thisfoo(e) {
      App.resetSessionTimer();
    });

    //session sleep timer
    App.lastActiveTime = new Date().getTime(); //int
    App.sessionSleepThread = window.setTimeout(function thisfoo(e) {
      var currentTime = new Date().getTime();
      var difference = currentTime - App.lastActiveTime - App.sleepCheckInterval;

      if (difference > App.sleepCheckThreshold) {
        App.vent.trigger('session:wokeUp');
      }

      App.lastActiveTime = currentTime;

      window.setTimeout(thisfoo, App.sleepCheckInterval);
    }, App.sleepCheckInterval);
  }

  return App;

});
