define(function (require) {
  var App = require('app');

  return Backbone.Collection.extend({

    url: function () {
      return App.Resources.get('patients').href;
    },
    parse: function (response) {
      if (typeof response.paging !== 'undefined') {
        return _.sortBy(response.patient, 'displayName');
      }
      App.Resources.store.put('patientCollection', response);
      return response;
    },
    myListFetch: function () {
      var restURL = App.Resources.get('patient-list').href + '?facilityId=' + App.Resources.user.vistaLocation;

      this.fetch({
        url: restURL,
        dataType: 'json',
        reset: true,
        success: function () {
          App.vent.trigger('myList:shown');
          App.vent.trigger('filtered');
        }
      }).fail(function (jqXHR) {
        if (jqXHR.status === 400) {
          App.vent.trigger('patient-list:error', $.parseJSON(jqXHR.responseText).message);
        }
      });
    },
    patientSearchResults: function (input) {
      var data = {};

      if (input.trim() !== '') {
        data.searchstring = encodeURIComponent(input.trim());
      }

      var searchString = '?';
      _.forEach(_.keys(data), function (key) {
        searchString += key + '=' + data[key] + '&';
      });

      this.fetch({
        url: this.url() + searchString,
        dataType: 'json',
        reset: true
      }).fail(function (jqXHR) {
        if (jqXHR.status === 400 || 403) {
          App.vent.trigger('patients:error', $.parseJSON(jqXHR.responseText).message);
        }
      });
    }

  });
});
