define(function (require) {
  var App = require('app');
  var HeaderView = require('header-view');
  var FooterView = require('footer-view');
  var EulaView = require('eula-view');
  var SplashView = require('splash-view');
  var SubfooterView = require('subfooter-view');
  var HomeView = require('home-view');
  var MyListView = require('my-list-view');
  var EditPatientView = require('edit-patient-view');
  var CallOutlinesView = require('call-outlines-view');
  var LogoView = require('logo-view');
  var AboutView = require('about-view');
  var LandingView = require('landing-view');
  var HeaderTitleView = require('header-title');
  var FiltersView = require('filters-view');
  var PatientListCollectionView = require('patient-list-collection-view');
  var PatientDetailLayoutView = require('patient-detail-view');
  var TimelineCompositeView = require('timeline-composite-view');
  var ProfileItemView = require('profile-item-view');
  var NotesItemView = require('notes-item-view');
  var InfoPatientDetailItemView = require('info-patient-detail-view');
  var SEDView = require('sed-view');
  var SEDCollection = require('sed-collection');
  var Patients = require('patients');
  var Patient = require('patient-information');

  return Backbone.Marionette.Controller.extend({

    initialize: function () {
      App.views = {};
      App.collections = {};
      App.views.header = new HeaderView({logo: LogoView});
      App.views.footer = new FooterView();
      App.footerRegion.show(App.views.footer);
      App.headerRegion.show(App.views.header);
    },
    showIndex: function () {
      App.goToTop();
      App.views.home = new HomeView();
      App.contentRegion.show(App.views.home);
      App.views.header.showLogo();
      App.views.header.title.close();

      if (App.Resources.isLoggedIn()) {
        App.views.home.upperRegion.show(new LandingView());
      } else {
        App.views.home.upperRegion.show(new SplashView());
      }

      App.views.home.lowerRegion.show(new SubfooterView());
    },
    showEula: function () {
      App.bodyRegion.show(new EulaView());
    },
    showEditPatient: function (model) {
      App.goToTop();
      if (!App.Resources.isLoggedIn()) {
        window.location.replace('/mcc-support/#home');
      } else {
        App.views.header.hideLogo();
        App.views.header.title.show(new HeaderTitleView({title1: 'Edit', title2: 'Patient'}));
        if (model) {
          App.contentRegion.show(new EditPatientView({model: model}));
        } else {
          App.contentRegion.show(new EditPatientView({}));
        }
      }
    },
    showMyPatient: function (model) {
      App.goToTop();
      if (!App.Resources.isLoggedIn()) {
        window.location.replace('/mcc-support/#home');
      } else {
        App.views.header.hideLogo();
        App.views.header.title.show(new HeaderTitleView({title1: 'Patient', title2: 'Detail'}));
        App.views.detail = new PatientDetailLayoutView();
        App.contentRegion.show(App.views.detail);

        App.views.detail.infoDetail.show(new InfoPatientDetailItemView({model: model}));
        App.views.detail.timeline.show(new TimelineCompositeView({model: model}));
        App.views.detail.profile.show(new ProfileItemView({model: model}));
        App.views.detail.notes.show(new NotesItemView({model: model}));
      }

    },
    showMyList: function (option) {
      App.goToTop();
      if (!App.Resources.isLoggedIn()) {
        window.location.replace('/mcc-support/#home');
      } else {
        var defaultOption = 2;
        App.views.myList = new MyListView();
        App.views.header.hideLogo();
        App.views.header.title.show(new HeaderTitleView({title1: 'Maternity', title2: 'Patients'}));
        App.contentRegion.show(App.views.myList);
        App.views.myList.filtersRegion.show(new FiltersView({
          'sortByOption': option || defaultOption
        }));
        App.views.myList.patientsRegion.show(new PatientListCollectionView({
          'sortByOption': option || defaultOption
        }));

      }
    },
    showCallOutlines: function (id) {
      App.views.header.hideLogo();
      App.views.header.title.show(new HeaderTitleView({title1: 'Call', title2: 'Outlines'}));
      switch (id) {
        case '1':
          window.open(
            './content/MCCSupport.pdf#page=47',
            '_blank');
          break;
        case '2':
          window.open(
            './content/MCCSupport.pdf#page=48',
            '_blank');
          break;
        case '3':
          window.open(
            './content/MCCSupport.pdf#page=49',
            '_blank');
          break;
        case '4':
          window.open(
            './content/MCCSupport.pdf#page=50',
            '_blank');
          break;
        case '5':
          window.open(
            './content/MCCSupport.pdf#page=51',
            '_blank');
          break;
        case '6':
          window.open(
            './content/MCCSupport.pdf#page=52',
            '_blank');
          break;
        case '7':
          window.open(
            './content/MCCSupport.pdf#page=54',
            '_blank');
          break;
        default:
          App.contentRegion.show(new CallOutlinesView());
      }
    },
    showAbout: function () {
      App.views.header.hideLogo();
      App.views.header.title.show(new HeaderTitleView({title1: 'About', title2: ''}));
      App.contentRegion.show(new AboutView());
    },
    showSED: function () {
      App.collections.sedCollection = new SEDCollection();
      App.views.header.hideLogo();
      App.views.header.title.show(new HeaderTitleView({title1: 'Patient Entered', title2: 'Data'}));
      App.collections.sedCollection.fetch({
        reset: true,
        success: function (data) {
          App.views.sedView = new SEDView(data.models);
          App.contentRegion.show(App.views.sedView);
        }
      });
    }
  });
});
