define(function (require) {
  var App = require('app');
  var ExitPopupView = require('exit-popup-view');
  var AddPatientView = require('add-patient-view');
  var PatientSearchFormView = require('patient-search-form');
  var PatientSearchDropListView = require('patient-search-list');
  var PopupHandleView = require('popup-handle-view');
  var PatientPopupView = require('patient-information-popup');

  var PatientImportantInfoView = require('patient-information-subview');
  var EditNotesItemView = require('edit-notes-item-view');
  var AddCustomCallView = require('add-custom-call-item-view');
  var DeleteConfirmationView = require('delete-confirmation-item-view');

  var SessionWarningItemView = require('session-warning-item-view');
  var SessionEndedItemView   = require('session-ended-item-view');

  var AjaxTimeoutView = require('ajax-timeout-view');
  var AjaxDisconnectView = require('ajax-disconnect-view');

  return Backbone.Marionette.Controller.extend({

    initialize: function () {
      this.listenTo(App.vent, 'popup:show', this.callMethod);
      this.listenTo(App.vent, 'popup:hideAll', this.hideAll);
      this.listenTo(App.vent, 'popup:destroyAll', this.destroyAll);
      this.on('addPatient', this.showAddPatient);
      this.on('exitPopup', this.showExitPopup);
      this.on('patientPopup', this.showPatientPopup);

      this.on('showSessionEnded', this.showSessionEnded);
      this.on('showAjaxTimeout', this.showAjaxTimeout);
      this.on('showAjaxDisconnect', this.showAjaxDisconnect);
    },
    callMethod: function (target) {
      this.trigger(target);
    },
    hideAll: function () {
      $(App.popupRegion.$el).find('.modal').modal('hide');
      $('.modal-backdrop').remove();
    },
    destroyAll: function () {
      App.popupRegion.close();
    },
    showAddPatient: function () {
      var view = new AddPatientView({
        searchp: new PatientSearchFormView(),
        results: new PatientSearchDropListView()
      });
      App.popupRegion.show(new PopupHandleView({
        view: view,
        idNum : '1'
      }));
    },
    showExitPopup: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new ExitPopupView(),
        idNum : '2'
      }));
    },
    showPatientPopup: function (model) {
      App.popupRegion.show(new PopupHandleView({
        view: new PatientPopupView({model: model}),
        idNum : '3'
      }));
    },
    showNotes: function (model) {
      App.popupRegion.show(new PopupHandleView({
        view: new EditNotesItemView({
          model: model
        }),
        idNum : '4'
      }));
    },
    showCustomCall: function (model, callDetailModel) {
      App.popupRegion.show(new PopupHandleView({
        view: new AddCustomCallView({
          model: model,
          callDetailModel: callDetailModel,
          childView: new PatientImportantInfoView({
            model: model
          })
        }),
        idNum : '5'
      }));
    },
    showDeleteConfirmation: function (model) {
      App.popupRegion.show(new PopupHandleView({
        view: new DeleteConfirmationView({
          model: model
        }),
        idNum : '6'
      }));
    },
    showSessionWarning: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new SessionWarningItemView({
          timeLeft: App.showWarningInterval
        }),
        idNum : '7'
      }));
    },
    showSessionEnded: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new SessionEndedItemView(),
        idNum : '8'
      }));
    },
    showAjaxTimeout: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new AjaxTimeoutView(),
        idNum : '9'
      }));
    },
    showAjaxDisconnect: function () {
      App.popupRegion.show(new PopupHandleView({
        view: new AjaxDisconnectView(),
        idNum : '10'
      }));
    }
  });
});
