define(function (require) {
  var App = require('app');

  return Backbone.Model.extend({

    defaults: {
      'id': null,
      'mccId': '',
      'patientId': '',
      'displayName': '',
      'nameFirst': '',
      'nameLast': '',
      'address': '',
      'city': '',
      'state': '',
      'zip': '',
      'phone': '',
      'phoneType': 'Cell',
      'textEnabled': false,
      'email': '',
      'lmp': -2209057200000,
      'edd': -2209057200000,
      'eddSetBy': '',
      'mcc': '',
      'vaProvider': '',
      'vaFacility': '',
      'facilityId': '',
      'obProvider': '',
      'obContacts': [],
      'isPregnant': true,
      'remindersActive': true,
      'callDetails': [],
      'hasCustomCalls': false,
      'notes': '',
      'actualEndDate': -2209057200000,
      'pregOutcome': '',
      'endNotes': '',
      'archiveFlag': false
    },

    url: function () {
      return App.Resources.get('patient-detail').href;
    },
    patientFetch: function (patientId) {
      if (typeof patientId === 'undefined') {
        patientId = this.get('patientId');
      }
      var restURL = App.Resources.get('patient-detail').href + '?mccPatientId=' + patientId;

      this.fetch({
        url: restURL,
        dataType: 'json',
        reset: true,
        async: false
      }).fail(function (jqXHR) {
        if (jqXHR.status === 400) {
          App.vent.trigger('patient-details:error', $.parseJSON(jqXHR.responseText).message);
        }
      });
    },
    parse: function (response) {
      var patient = response;
      if (typeof response.link !== 'undefined') {
        this.Resources = App.Resources.instance(response.link);
        patient = {
          'displayName': response.displayName,
          'nameFirst': response.firstName,
          'nameLast': response.lastName,
          'textEnabled': response.textMessagingAllowed
        };
      }
      if (typeof response.patientIdentifier !== 'undefined') {
        patient.patientId = response.patientIdentifier.uniqueId;
        return patient;
      }
      return patient;
    },
    getNextCall : function () {
      var callArray = this.attributes.callDetails;
      var reminders = this.attributes.remindersActive;
      var emptyDetail = {
        id: -1,
        callType: 'null',
        date: -2209057200000,
        reason: 'All Calls Completed',
        callsCompleted: false,
        callsAttempted: false,
        notesEntered: false
      };
      if (!reminders) {
        callArray = this.attributes.callDetails.filter(function (obj) {
          return obj.callType === 'custom';
        });
      }

      if (callArray.length !== 0) {
        var now = Date.now();
        var min = - Math.abs((new Date(0, 0, 0)).valueOf());
        var output;
        var firstCall = _.find(callArray, function (call) { return parseInt(call.id, 10) === 1; });

        if (firstCall && reminders) {
          if (!(firstCall.callsCompleted || firstCall.callsAttempted)) {
            output = firstCall;
            return output;
          }
        }
        for (var d = 0 ; d < callArray.length ; d++) {
          var diff = now - callArray[d].date;
          if (diff >= min && !(callArray[d].callsCompleted || callArray[d].callsAttempted)) {
            output = callArray[d];
            min = diff;
          }
        }

        return output || emptyDetail;
      } else {
        emptyDetail.id = -2;
        return emptyDetail;
      }
    }
  });
});
