define(function () {

  var getJSON = function (path) {
    var response = $.ajax({url: path, dataType: 'json', async: false});
    return JSON.parse(response.responseText);
  };

  var HAResources = function (app, options) {
    options = options || {};
    this.vent = app.vent;
    this.user = {};
    this.directory = {};
    this.store = {
      pool: {},
      put: function (tag, objectFunc) {
        this.pool[tag] = objectFunc;
      },
      get: function (tag) {
        if (typeof this.pool[tag] === 'function') {
          this.pool[tag] = new this.pool[tag]();
        }
        return this.pool[tag];
      }
    };
    this.initialize.apply(this, arguments);
  };

  _.extend(HAResources.prototype, {
    initialize: function () {
      var path = this.getResourcePath();
      this.directory = getJSON(path).link;
      this.parseToken();
      this.fetch('public-user-session');

      this.user = (this.isLoggedIn()) ? this.store.get('public-user-session').mhpuser : {};
    },
    get: function (tag) {
      return _.first(this.directory.filter(function (obj) { return obj.title === tag; }));
    },
    getResourcePath: function () {
      return window.location.protocol + '//' + window.location.host + '/mcc-support-resources/rest/public/resource-directory';
    },
    getRedirectURI: function () {
      return '?redirect_uri=' + window.location.protocol + '//' + window.location.host + window.location.pathname;
    },
    fetch: function (tag) {
      this.store.put(tag, getJSON(this.get(tag).href));
    },
    instance: function (link) {
      return {
        directory: link,
        get: this.get
      };
    },
    parseToken: function () {
      var params = {}, queryString = location.search.substring(1),
        regex = /([^&=]+)=([^&]*)/g,
        m = regex.exec(queryString);
      if (m !== null) {
        params[decodeURIComponent(m[1])] = decodeURIComponent(m[2]);
      }

      var token = params.token;

      if (typeof token !== 'undefined' && token !== 'undefined' && token !== null && token !== 'null') {
        window.sessionStorage.token = JSON.stringify(token);
        window.location = window.location.protocol + '//' + window.location.host + window.location.pathname;
      }

      this.checkToken();
      var that = this;
      $(document).ajaxError(function (e, jqxhr, settings) {
        var alertMessage = function () {
          alert('unknown error at url: ' + settings.url + '\nstatus code: ' + jqxhr.status);
        };
        switch (jqxhr.status) {
          case 0:
          case 200:
          case 400:
          case 404:
            return;
          case 401:
            if (jqxhr.responseText.indexOf('invalid_token') > 0) {
              that.cleanUpSession();
              $.ajax(settings);
            }
            return;
          case 403:
            that.cleanUpSession();
            if (typeof device !== 'undefined') {
              var ref = window.open(that.get('login').get('href') + that.getRedirectURI(), '_blank', 'location=no,toolbar=no');
              ref.addEventListener('loadstart', function (e) {
                var token = e.url.split('token=')[1];
                if (token) {
                  window.sessionStorage.token = JSON.stringify(token);
                  that.checkToken();
                  window.location.reload();
                  ref.close();
                }
              });
            }
            else {
              window.location = that.get('login').get('href') + that.getRedirectURI();
            }
            return;
          case 500:
            //Call alert if stack trace is included // This represents a dev alert
            if (jqxhr.responseText.indexOf('java.lang') > 0) {
              alertMessage.call();
            }
            return;
          default:
            alertMessage.call();
        }
      });

    },
    checkToken: function () {
      var storedToken = window.sessionStorage.token;

      if (typeof storedToken !== 'undefined' && storedToken !== 'undefined' && storedToken !== null && storedToken !== 'null') {
        $.ajaxPrefilter(function (options, originalOptions, jqXHR) {
          var currentToken = JSON.parse(storedToken);
          jqXHR.setRequestHeader('Authorization', 'Bearer ' + currentToken);

        });

      } else {
        this.cleanUpSession();
      }
    },
    isLoggedIn: function () {
      return !!(this.store.get('public-user-session').mhpuser);
    },
    getUserName : function () {
      return this.user.displayName || '';
    },
    getUserFacility : function () {
      return this.user.facilityName;
    },
    getUserLocation : function () {
      return this.user.vistaLocation;
    },
    getAssigningAuthority : function () {
      return this.user.userIdentifier.assigningAuthority;
    },
    login: function () {
      window.location = this.get('login').href + this.getRedirectURI();
    },
    cleanUpSession: function () {
      window.sessionStorage.token = null;
      $.ajaxPrefilter(function (options, originalOptions, jqXHR) {
        jqXHR.setRequestHeader('Authorization', '');
      });
      document.cookie = encodeURIComponent('JSESSIONID') + '=deleted; expires=' + new Date(0).toUTCString();
    },
    getThisWeeksCallQueue: function () {
      var line = this.get('weekly-call-list').href + '?mccId=' + this.user.id;
      return line;
    },
    resetSessionTimer: function () {
      var path = this.get('user-session').href;
      $.ajax({url: path, dataType: 'json', async: false, global: false});
      return true;
    },
    getTimeToExpireInMilliSeconds: function () {
      var path = this.get('last-accesstime').href;
      var that = this;
      var response = $.ajax({url: path, dataType: 'json', async: false, global: false, error: function () {
        that.vent.trigger('session:ended');
      }});

      var result;
      if (response.status === 200) {
        var json = JSON.parse(response.responseText);
        result = json.timeToExpireInSeconds;
      } else {
        result = '0';
      }

      return parseInt(result) * 1000;
    }

  });

  return HAResources;
});
