define(function (require) {
  var App = require('app');
  var Patient = require('patient-information');
  var InfoSubview = require('patient-information-subview');
  var template = require('text!../html/editPatientView.html');
  var additionalTemplate = require('text!../html/editPatientAdditional.html');
  var CallOutline = require('text!../json/callOutline.json');
  var bootstrapDatepicker = require('bootstrap-datepicker');
  var maxContactRows = 5;


  return Backbone.Marionette.Layout.extend({
    id: 'editPatient',
    tagName: 'div',
    className: 'otherView',
    template: _.template(template),
    additionalTemplate: _.template(additionalTemplate),

    regions: {
      subviewr: '#patient-info-subview'
    },

    events: {
      'click #save-patient-details': 'savePatient',
      'click #delete-patient-details': 'stopTrackingPatient',
      'click .info-add': 'addContactInfo',
      'click .info-remove': 'removeContactInfo',
      'click #is-pregnant': 'toggleIsPregnantText',
      'click #call-reminders': 'toggleCallRemindersText',
      'change .dateform': 'validateDateField'

    },
    initialize: function (options) {
      if (options.model) {
        this.model = options.model;
      } else if (window.sessionStorage.getItem('patientId')) {
        this.model = new Patient();
        this.model.patientFetch(window.sessionStorage.getItem('patientId'));
        if (this.model.get('patientId').length === 0) {
          window.location.replace('/mcc-support/#home');
        }
      } else {
        this.model = new Patient();
      }
      window.sessionStorage.setItem('patientId', this.model.get('patientId'));
      if (typeof this.model.get('mccId') === 'undefined' || !this.model.get('mccId')) {
        this.model.set('mcc', App.Resources.getUserName());
        this.model.set('vaFacility', App.Resources.getUserFacility());
        this.model.set('facilityId', App.Resources.getUserLocation());
      }
      this.subview = new InfoSubview({
        model: this.model
      });
    },
    onRender: function () {
      this.subviewr.show(this.subview);

    },
    addContactInfo: function (e) {
      var $contactInfoRows = $('.ob-provider-info-row');
      var x = $contactInfoRows.length;
      if (x < maxContactRows) {
        $contactInfoRows.last().after(this.additionalTemplate).trigger('create');
      }
      if ((x + 1) >= maxContactRows) {
        $(e.currentTarget).attr('disabled', 'disabled');
      }
    },
    removeContactInfo: function (e) {
      var $button = $(e.currentTarget);
      var $contactInfoRows = $button.parents('.ob-provider-info-row');
      $contactInfoRows.last().remove();
      var x = $contactInfoRows.length;
      if (x < maxContactRows) {
        $('.info-add').removeAttr('disabled');
      }
    },
    toggleIsPregnantText: function (e) {
      var $button = $(e.target);
      if ($button.hasClass('active')) {
        $button.contents().filter(function () {
          return this.nodeType === 3;
        }).last().replaceWith('No');
      } else {
        $button.contents().filter(function () {
          return this.nodeType === 3;
        }).last().replaceWith('Yes');
      }
    },
    toggleCallRemindersText: function (e) {
      var $button = $(e.target);
      if ($button.hasClass('active')) { $button.contents().filter(function () {
          return this.nodeType === 3;
        }).last().replaceWith('Off');
      } else {
        $button.contents().filter(function () {
          return this.nodeType === 3;
        }).last().replaceWith('On');
      }
    },
    validateDateField: function (e) {
      var $datBox = $(e.currentTarget);
      var isValid = App.router.dates.dateFieldIsValid($datBox);
      if (isValid && $datBox.attr('id') === 'last-menstrual-period') {
        this.updateEDD();
      }
    },
    updateEDD: function () {
      var $lmpBox = $('#last-menstrual-period');
      if ($lmpBox.length === 0 || $('#ultrasound').prop('checked')) {
        return;
      } else {
        $lmpBox.removeClass('changed');
      }
      var dat = App.router.dates.validateDate($lmpBox.val());
      if (dat) {
        var datAdd = App.router.dates.addDays(dat, App.router.dates.getGestation());
        var datOut = App.router.dates.convertDate(datAdd);
        var eddBox = $('#estimated-due-date');
        eddBox.val(datOut);
        eddBox.change();
        eddBox.siblings('.datepicker').datepicker('update', dat);
        $('#lmp').prop('checked', true);
      }
    },
    onShow: function () {
      $('.datepicker').datepicker({
        todayBtn: 'linked',
        todayHighlight: true,
        autoclose: true,
        orientation: 'top right'
      }).on('changeDate', function (e) {
        App.router.dates.changeDate(e);
      });
      $('#mcc').val(this.model.get('mcc'));
      $('#va-primary-facility').val(this.model.get('vaFacility'));
    },
    leavePage: function (model) {
      App.router.navigate('my-patient');
      App.router.controller.showMyPatient(model);
    },
    stopTrackingPatient: function (e) {
      e.preventDefault();
      App.router.popups.showDeleteConfirmation(this.model);
    },
    savePatient: function (e) {
      e.preventDefault();
      var that = this;
      var formInputs = $('form').serializeArray();
      formInputs = formInputs.concat(
        $('form input[type=checkbox]:not(:checked)').map(function () {
          return { 'name': this.name, 'value': 'off' };
        }).get()
      );
      var obContactList = [];
      var eddChanged = false;
      var actualEndDateChanged = false;
      var patient = this.model;
      for (var i = 0; i < formInputs.length; i++) {
        if (_.contains(['isPregnant', 'remindersActive', 'textEnabled'], formInputs[i].name)) {
          patient.set(formInputs[i].name, formInputs[i].value === 'on');
        }
        else if (_.contains(['lmp', 'edd', 'actualEndDate'], formInputs[i].name)) {
          var dateOut = App.router.dates.validateDate(formInputs[i].value);
          if (!(dateOut instanceof Date)) {
            dateOut = App.router.dates.getDefaultDate();
          }
          if (dateOut.getTime() !== patient.get(formInputs[i].name)) {
            if (formInputs[i].name === 'edd') {
              eddChanged = true;
            }
            else if (formInputs[i].name === 'actualEndDate') {
              actualEndDateChanged = true;
            }
            patient.set(formInputs[i].name, dateOut);
          }
        }
        else if (formInputs[i].name === 'contactInfo') {
          var obContactObj = {};
          obContactObj[formInputs[i].name] = formInputs[i].value;
          obContactObj[formInputs[i + 1].name] = formInputs[i + 1].value;
          obContactList.push(obContactObj);
          i++;
        }
        else {
          patient.set(formInputs[i].name, formInputs[i].value);
        }
      }
      patient.set('obContacts', obContactList);
      /*patient.set('dateOfBirth', App.router.dates.validateDate(patient.get('dateOfBirth')));*/

      var callDetails = [];
      if (typeof patient.get('mccId') === 'undefined' || !patient.get('mccId')) {
        patient.set('mccId', App.Resources.user.userIdentifier.uniqueId);
        callDetails = callDetails.concat(this.generateFirstContact());
      }
      if (patient.get('edd') !== 'undefined' && !App.router.dates.isDefaultDate(patient.get('edd')) &&
        eddChanged) {
        callDetails = callDetails.concat(this.generateCallQueue(patient.get('edd')));
      }

      if (callDetails.length > 0) {
        callDetails = this.compareCallDetailsArrays(patient.get('callDetails'), callDetails);
        callDetails = _.sortBy(callDetails, function (detail) {
          return App.router.dates.validateDate(detail.date).getTime();
        });
        patient.set('callDetails', callDetails);
      }

      if (patient.get('actualEndDate') !== 'undefined' &&
        !App.router.dates.isDefaultDate(patient.get('actualEndDate')) && !eddChanged && actualEndDateChanged) {
        this.updatePostPartum(patient.get('actualEndDate'));
      }
      App.vent.trigger('updateNextCall');
      patient.save(null, {
        dataType: 'text',
        success: function (model) {
          model.patientFetch();
          that.leavePage(model);
        },
        wait: true
      });
    },
    generateFirstContact: function () {
      return [{
        id: '1',
        callType: 'outline',
        date: App.router.dates.getZeroHourDate(),
        reason: 'First Contact',
        callsCompleted: false,
        callsAttempted: false,
        notesEntered: false
      }];
    },
    updatePostPartum: function (aed) {
      var calls = this.model.get('callDetails');
      var assumedID = '7';
      var assumedReason = '6 Weeks Postpartum';
      var updatedPPC = App.router.dates.addDays(aed, 6 * 7);
      var i = 0;
      for (i ; i < calls.length ; i++) {
        if (calls[i].id === assumedID || calls[i].reason === assumedReason) {
          calls[i].date = updatedPPC;
          break;
        }
      }
      calls = _.sortBy(calls, function (detail) {
        return App.router.dates.validateDate(detail.date).getTime();
      });
      this.model.set('callDetails', calls);

    },
    generateCallQueue: function (edd) {
      var calls = JSON.parse(CallOutline);
      var output = [];
      for (var i = 0; i < calls.length; i++) {
        var date = null;
        switch (calls[i].metric) {
          case 'currentDate':
            date = App.router.dates.addDays(App.router.dates.getZeroHourDate(), calls[i].deltaDays);
            break;
          case 'edd':
            date = App.router.dates.addDays(edd, calls[i].deltaDays);
            break;
        }
        //if ((+date >= +new Date())) {} TODO:disabled until SME confirmation.
        output.push({
          id: (i + 2).toString(),
          callType: 'outline',
          date: date,
          reason: calls[i].reason,
          callsCompleted: false,
          callsAttempted: false,
          notesEntered: false
        });
      }
      return output;
    },
    compareCallDetailsArrays: function (currCalls, newCalls) {
      var j = 0;
      var i = 0;
      for (j; j < newCalls.length; j++) {
        var callMatched = false;
        for (i; i < currCalls.length; i++) {
          if (currCalls[i].id === newCalls[j].id) {
            callMatched = true;
            break;
          }
        }
        if (!callMatched) {
          currCalls.push(newCalls[j]);
        }
        else if (!currCalls[i].callsCompleted && !currCalls[i].callsAttempted && newCalls[j].id !== '1') {
          currCalls[i] = newCalls[j];
        }
      }
      return currCalls;
    },
    templateHelpers: {
      dateFormatted: function (datestr) {
        if (datestr && !App.router.dates.isDefaultDate(datestr)) {
          return App.router.dates.convertDate(new Date(datestr));
        } else {
          return '';
        }
      }
    }
  });
});
