define(function (require) {
  var App = require('app');
  var template = require('text!../html/myListView.html');

  return Backbone.Marionette.Layout.extend({
    id: 'myList',
    tagname: 'div',
    className: 'otherView',
    template: _.template(template),

    regions: {
      'filtersRegion' : '#filters-items',
      'patientsRegion' : '#patient-list-collection'
    },
    events: {
      'click #add-patient-btn' : 'launchModal',
      'click #custom-calls-view' : 'filterCustoms',
      'change #view-filter' : 'filterViewOptions',
      'click #call-queue-view' : 'sortByCall',
      'click #estimated-time-view' : 'sortByEDD',
      'click #patient-name-view' : 'sortByName'
    },
    launchModal : function (e) {
      e.preventDefault();
      App.vent.trigger('popup:show', 'addPatient');
    },
    filterViewOptions : function () {
      var collectionListView = App.views.myList.patientsRegion.currentView;
      collectionListView.collection.reset(App.Resources.store.get('patientCollection'));
      var selectedView = $('#view-filter option:selected').val();
      if (selectedView !== '0') {
        var filteredPatients = collectionListView.collection.where({
          mccId: selectedView
        });

        collectionListView.collection.set(filteredPatients);
      }
      App.vent.trigger('myList:shown');
      collectionListView.render();
    },
    initialize : function () {
      App.vent.on('myList:shown', this.updateResultsCount);
      App.vent.once('filtered', this.addMccs);
      App.vent.on('filter:done', this.selectCurrentMcc);
      App.vent.on('filter:selected', this.filterViewOptions);
    },
    selectCurrentMcc : function () {
      $('#view-filter').val(App.Resources.user.userIdentifier.uniqueId);
      App.vent.trigger('filter:selected');
    },
    addMccs : function () {
      var collectionListView = App.views.myList.patientsRegion.currentView;
      var mccCollection = collectionListView.collection;
      var grouped = mccCollection.groupBy('mccId');

      _.each(grouped, function (group) {
        $('#view-filter')
          .append($('<option></option>')
            .attr('value', group[0].get('mccId'))
            .text(group[0].get('mcc')));
      });
      App.vent.trigger('filter:done');
    },
    updateResultsCount : function () {
      var spanTag = $('#resultsCount');
      var collectionListView = App.views.myList.patientsRegion.currentView;
      if (spanTag.length > 0) {
        spanTag.text(collectionListView.collection.length);
      }
    },
    sortByName : function () {
      var collectionListView = App.views.myList.patientsRegion.currentView;
      collectionListView.collection.comparator = collectionListView.sortByPatientName;
      collectionListView.collection.sort();
      collectionListView.render();

    },
    sortByCall : function () {
      var collectionListView = App.views.myList.patientsRegion.currentView;
      collectionListView.collection.comparator = collectionListView.sortByCall;
      collectionListView.collection.sort();
      collectionListView.render();

    },
    sortByEDD : function () {
      var collectionListView = App.views.myList.patientsRegion.currentView;
      collectionListView.collection.comparator = collectionListView.sortByEDD;
      collectionListView.collection.sort();
      collectionListView.render();

    },

    filterCustoms : function () {
      var collectionListView = App.views.myList.patientsRegion.currentView;
      collectionListView.collection.reset(App.Resources.store.get('patientCollection'));
      var customCheck = $('#custom-calls-view').prop('checked');

      if (customCheck === true) {
        var filteredPatients = collectionListView.collection.where({
          hasCustomCalls: customCheck
        });

        collectionListView.collection.set(filteredPatients);
      }
      App.vent.trigger('myList:shown');
      collectionListView.render();
    }

  });
});
