define(function (require) {
  var App = require('app');
  var Patient = require('patient-information');
  var template = require('text!../html/profileItemView.html');
  var InfoSubview = require('patient-information-subview');
  return Backbone.Marionette.Layout.extend({
    id: 'profile-item-view',
    tagName: 'div',
    className: 'corners-rounded',
    template: _.template(template),
    regions: {
      subviewr: '#pii-box'
    },
    events: {
      'click #edit-patient': 'showEditPatient',
      'click #custom-call': 'launchPopup'
    },
    initialize: function (options) {
      var Patient = require('patient-information');
      if (options.model) {
        this.model = options.model;
      } else if (window.sessionStorage.getItem('patientId')) {
        this.model = new Patient();
        this.model.patientFetch(window.sessionStorage.getItem('patientId'));
      }
      window.sessionStorage.setItem('patientId', this.model.get('patientId'));
      this.subview = new InfoSubview({
        model: this.model
      });
    },
    onRender: function () {
      this.subviewr.show(this.subview);
    },
    showEditPatient: function (e) {
      e.preventDefault();
      App.router.navigate('edit-patient', {trigger: true});
      App.router.controller.showEditPatient(this.model);
    },
    launchPopup: function (e) {
      e.preventDefault();
      App.router.popups.showCustomCall(this.model, null);
    },
    templateHelpers: {
      dobFormatted: function () {
        return App.router.dates.convertDate(new Date(this.dateOfBirth));
      },
      pregWeek: function () {
        startDate = (typeof startDate === 'undefined') ? App.router.dates.getZeroHourDate() : new Date(startDate);
        if (!App.router.dates.isDefaultDate(this.edd)) {
          var date = App.router.dates.addDays(new Date(this.edd), (-App.router.dates.getGestation()));
          var preg = App.router.dates.getWeekAndDays(date, startDate);
          if (preg.weeks >= 44) {
            return 'Outside of the expected range of pregnancy.';
          } else {
            if (!preg.string) {
              return '';
            } else {
              return preg.string + ' Pregnant';
            }
          }
        } else {
          return '';
        }
      },
      getPrimaryPhone: function () {
        var line = '' + this.phone;
        if (this.textEnabled) {
          line += ' (Text Enabled)';
        }
        return line;
      },
      getAge: function (date) {
        return App.router.dates.calculateAge(App.router.dates.validateDate(new Date(date)));
      }
    }
  });
});
