define(function (require) {
  var App = require('app');
  var template = require('text!../html/timelineCompositeView.html');
  var TimelineItemView = require('timeline-item-view');
  var TimelineDetailsItemView = require('timeline-details-item-view');

  var Patient = require('patient-information');

  return Backbone.Marionette.CompositeView.extend({
    id: 'timeline-container',
    tagName: 'div',
    className: '',
    template: _.template(template),

    itemView: TimelineItemView,
    itemViewContainer: '#timeline',

    events: {
      'click .timeline-arrow-right': 'nextPage',
      'keypress .timeline-arrow-right': 'nextPage',
      'click .timeline-arrow-left': 'prevPage',
      'keypress .timeline-arrow-left': 'prevPage'
    },
    initialize : function (options) {

      if (options.model) {
        this.model = options.model;
      } else if (window.sessionStorage.getItem('patientId')) {
        this.model = new Patient();
        this.model.patientFetch(window.sessionStorage.getItem('patientId'));
      }
      window.sessionStorage.setItem('patientId', this.model.get('patientId'));

      var model = this.model;
      this.callArray = this.sortCalls(model);

      var nxtCall = this.updateNextCall();
      this.showTimelineDetails(new Backbone.Model(nxtCall));

      this.cursor = this.setCursor(nxtCall.id);
      this.on('render:array', this.renderArray);
      this.on('showTimeline', this.showTimeline);
      this.trigger('render:array');
      this.listenTo(App.vent, 'timeline:checkbox', this.updateCallDetail);
      this.listenTo(App.vent, 'timeline:update', this.timelineRefresh);

      this.itemViewOptions = {nextCall: this.nextCallID};
      this.trigger('showTimeline');

      this.updateArrows = function () {
        var $rightArrow = $('.timeline-arrow-right');
        var $leftArrow = $('.timeline-arrow-left');
        if ($rightArrow.length > 0 && this.callArray.length === 1) {
          $rightArrow.addClass('disabled').removeAttr('tabindex').attr('aria-hidden', 'true');
        }
        if ($leftArrow.length > 0 && this.cursor === 0) {
          $leftArrow.addClass('disabled').removeAttr('tabindex').attr('aria-hidden', 'true');
        }
        if ($rightArrow.length > 0 && this.cursor === this.callArray.length - 1) {
          $rightArrow.addClass('disabled').removeAttr('tabindex').attr('aria-hidden', 'true');
        }

        this.trigger('showTimeline');
      };
    },
    showTimeline: function () {
      if (this.callArray.length === 0) {
        $('#timeline-region').hide();
        $('#timeline-details-region').hide();
      } else {
        $('#timeline-region').show();
        $('#timeline-details-region').show();
      }
    },
    timelineRefresh: function () {

      var that = this;
      var url = this.model.url() + '?mccPatientId=' + this.model.get('patientId');
      this.model.fetch({url: url, success: function () {
        that.callArray = that.sortCalls(that.model);
        that.updateNextCall();
        that.cursor = that.setCursor(App.activeCallID);
        that.trigger('render:array');

      }});

    },
    setCursor: function (findMe) {
      var output = '0';
      if (findMe < 0) {
        findMe = App.activeCallID;
      }
      if (this.callArray.length !== 0) {
        _.each(this.model.get('callDetails'), function (call, index) {
          if (call.id === findMe) {
            output = index;
          }
        });
        return Math.floor(parseInt(output) / 7);
      } else {
        return 0;
      }

    },
    updateCallDetail: function (id, key, val) {
      _.each(this.model.get('callDetails'), function (call) {
        if (call.id === id.toString()) { call[key] = val; }
      });
      if (key === 'callsCompleted' || key === 'callsAttempted') {
        this.updateNextCall();
      }
      this.callArray = this.sortCalls(this.model);
      this.model.save({}, { global: false });
    },
    focusLeft: function () {
      var $timeDiv = $('#timeline');
      if ($timeDiv.length > 0) {
        var $leftMostDateNode = $timeDiv.children().first();
        setTimeout(function () {
          $leftMostDateNode.find('a').focus();
        }, 200);
      }
    },
    onShow: function () {
      this.on('render', this.updateArrows);
      this.on('render', this.focusLeft);
      this.updateArrows();
    },
    sortCalls: function (model) {
      var sorted = [], output = [];
      var callArray = model.get('callDetails');
      if (!this.model.get('remindersActive')) {
        callArray = $.grep(callArray, function (v) {
          return v.callType !== 'outline';
        });
      }
      for (var j = 0; j < callArray.length; j += 7) {
        sorted = callArray.slice(j, j + 7);
        output.push(sorted);
      }
      return output;
    },
    updateNextCall: function () {
      var callDetail = this.model.getNextCall();

      this.nextCallID = callDetail.id;
      App.activeCallID = App.activeCallID || callDetail.id;
      this.itemViewOptions = {nextCall: this.nextCallID};

      App.vent.trigger('timeline:nextCall', callDetail.id.toString());
      App.vent.trigger('updateNextCall', callDetail);

      if (callDetail.id < 0) {
        callDetail = _.last(this.model.get('callDetails'));
        App.activeCallID = callDetail.id;
      }

      return callDetail;
    },
    showTimelineDetails: function (model) {
      App.views.detail.timelineDetails.show(new TimelineDetailsItemView({model: model, TimelineCompositeView: this, nextCallID : this.nextCallID}));
    },
    renderDetailsSubView: function (model) {
      App.views.detail.timelineDetails.currentView.showTimelineDetails(model);
    },
    nextPage : function (e) {
      e.preventDefault();
      this.cursor += (this.cursor === this.callArray.length - 1) ? 0: 1;
      if (!this.$el.find('.timeline-arrow-right').hasClass('disabled')) {
        this.trigger('render:array');
      }
      if (this.cursor > 0) {
        this.$el.find('.timeline-arrow-left').removeClass('disabled').removeAttr('aria-hidden').attr('tabindex', '0');
      }
      if (this.cursor === this.callArray.length - 1) {
        this.$el.find('.timeline-arrow-right').addClass('disabled').removeAttr('tabindex').attr('aria-hidden', 'true');
      }
    },
    prevPage: function (e) {
      e.preventDefault();
      this.cursor -= (this.cursor === 0) ? 0 : 1;
      if (!this.$el.find('.timeline-arrow-left').hasClass('disabled')) {
        this.trigger('render:array');
      }
      if (this.cursor < this.callArray.length - 1) {
        this.$el.find('.timeline-arrow-right').removeClass('disabled').removeAttr('aria-hidden').attr('tabindex', '0');
      }
      if (this.cursor === 0) {
        this.$el.find('.timeline-arrow-left').addClass('disabled').removeAttr('tabindex').attr('aria-hidden', 'true');
      }
    },
    renderArray: function () {
        this.collection = new Backbone.Collection(this.callArray[this.cursor]);
        this.render();
      }
  });
});
