define(function (require) {
  var App = require('app');
  var template = require('text!../html/timelineDetailsItemView.html');
  var nextCallID = '';

  return Backbone.Marionette.ItemView.extend({
    id: 'timeline-details',
    tagName: 'div',
    className: '',
    template: _.template(template),
    events: {
      'click [type="checkbox"]': 'updateDetail',
      'click #call-detail-info-btn': 'showCallDetailsInformation'
    },
    initialize: function (options) {
      nextCallID = options.nextCallID;
      this.TimelineCompositeView = options.TimelineCompositeView;
      this.listenTo(App.vent, 'timeline:details', this.renderDetails);
      this.listenTo(App.vent, 'timeline:updateDetailsModel', this.updateModel);
      this.listenTo(App.vent, 'timeline:nextCall', this.nextCallActive);
      this.listenTo(this.TimelineCompositeView, 'show', this.onRender);
    },
    updateModel: function (model) {
      this.model = model;
    },
    renderDetails: function (model) {
      this.model = model;
      this.render();
      var focus = function () {
        var $focus = $('#call-detail-info-btn');
        $focus.focus();
      };
      setTimeout(focus, 150);
    },
    updateDetail: function (e) {
      switch (e.currentTarget.id) {
        case 'callsCompleted':
          if (e.currentTarget.checked) {
            this.$el.find('#callsAttempted').attr('disabled', true);
          } else {
            this.$el.find('#callsAttempted').removeAttr('disabled');
          }
          break;
        case 'callsAttempted':
          if (e.currentTarget.checked) {
            this.$el.find('#callsCompleted').attr('disabled', true);
          } else {
            this.$el.find('#callsCompleted').removeAttr('disabled');
          }
          break;
      }
      this.model.set(e.currentTarget.id, e.currentTarget.checked);
      App.vent.trigger('timeline:checkbox', this.model.get('id'), e.currentTarget.id, e.currentTarget.checked);
      var focus = function () {
        var $focus = $('.selected');
        $focus.focus();
      };
      setTimeout(focus, 150);
    },
    nextCallActive: function (id) {
      nextCallID = id;
      this.render();
    },
    templateHelpers: {
      isEmpty: function () {
        if (nextCallID === -2) {
          return false;
        }
        else {
          return true;
        }
      },
      isNextCall: function (id) {
        return (id.toString() === nextCallID) ? 'Next Call' : '';
      },
      getDate: function (date) {
        if (nextCallID === -1) {
          return '';
        } else {
          return App.router.dates.convertDate(new Date(date));
        }

      },
      getType: function (callType, id) {
        switch (callType) {
          case 'outline':
            return 'Call Outline #' + id;
          case 'custom':
            return 'Custom Call';
        }
      }
    },
    showCallDetailsInformation: function (e) {
      e.preventDefault();
      var id = $('#call-detail-info-btn').attr('calldetailid');
      var callDetails = App.views.detail.timeline.currentView.model.get('callDetails');
      var callDetailObjects  = new Backbone.Collection(callDetails).where({id: id});

      if (callDetailObjects.length > 0) {
        var callDetailObject = callDetailObjects.pop();
        if (callDetailObject.get('callType') === 'outline') {
          App.router.controller.showCallOutlines(id);
        } else {
          App.router.popups.showCustomCall(App.views.detail.timeline.currentView.model, callDetailObject);
        }
      }
    },
    showCallOutline: function (e) {
      var id = $('#call-detail-info-btn').attr('calldetailid');
      App.router.controller.showCallOutlines(id);
    }
  });
});
