define(function (require) {
  var App = require('app');
  var template = require('text!../html/patientInformationSubview.html');
  var PII = require('patient-sensitive-information');
  var loading;
  var eloading;

  return Backbone.Marionette.ItemView.extend({
    id: '',
    tagName: 'div',
    className: 'patient-information',
    template: _.template(template),

    initialize: function (options) {
      this.model = options.model || {};
      loading = $('.ajax-loading').parent();
      eloading = $('.ajax-loading').detach();
    },
    onShow: function () {
      var pid = this.model.get('patientId');
      var authorityId = App.Resources.getAssigningAuthority();
      var url = App.Resources.get('patient').href + '/' + authorityId + '/' + pid;
      var pii = new PII();

      var that = this;
      pii.fetch({
        url: url,
        success: function () {
          that.model = pii;
          that.render();
          $('.pii-loading').hide();
          $('#pii-details').show();
        }
      });
      loading.append(eloading);
    },
    templateHelpers: {
      getSSN: function () {
        return this.ssn || '';
      },
      getDOB: function () {
        return this.dateOfBirth || '';
      },
      getGender: function () {
        return this.gender || '';
      },
      getDate: function (date) {
        if (date.length > 0) {
          return App.router.dates.convertDate(new Date(date));
        } else {
          return '';
        }
      },
      getAge: function (date) {
        if (date.length > 0) {
          return App.router.dates.calculateAge(App.router.dates.validateDate(new Date(date)));
        } else {
          return '';
        }
      },
      getFormattedAge: function (date) {
        if (date.length > 0) {
          return '(' + App.router.dates.calculateAge(App.router.dates.validateDate(new Date(date)))  + ' y o)';
        } else {
          return '';
        }

      }
    }

  });
});
