define(function (require) {
  var App = require('app');
  var template = require('text!../html/addCustomCallItemView.html');
  var bootstrapDatepicker = require('bootstrap-datepicker');
  var InfoSubview = require('patient-information-subview');

  return Backbone.Marionette.Layout.extend({
    tagName: 'div',
    id: 'addCustomCallItemView',
    template: _.template(template),
    initialize: function (args) {
      this.model = args.model;

      this.callDetail = args.callDetailModel;

      if (this.callDetail) {
        this.callDetailReason = this.callDetail.get('reason') || '';
        this.callDetailDate = App.router.dates.convertDate(new Date(this.callDetail.get('date')));
      } else {
        this.callDetailReason = '';
        this.callDetailDate = '';
      }
      this.subview = new InfoSubview({
        model: this.model
      });
      this.listenTo(App.vent, 'popup:shown', this.renderDate);
    },
    regions: {
      subviewr: '#pii-region'
    },
    events: {
      'click #exit-btn': 'closePopup',
      'click #save-custom-call': 'saveCall',
      'change #custom-call-date' : 'validateForm',
      'keyup #custom-call-data': 'validateForm'
    },
    renderDate: function () {
      $('.datepicker').datepicker({
        todayBtn: 'linked',
        todayHighlight: true,
        autoclose: true,
        orientation: 'top right'
      }).on('changeDate', function (e) {
        App.router.dates.changeDate(e);
      });
    },
    onRender: function () {
      this.subviewr.show(this.subview);
    },
    onShow: function () {
      var txtBoxRegion = this.$el.find('.txt-box');
      if (txtBoxRegion.length > 0) {
        txtBoxRegion.keyup(this.validateTextBox);
        txtBoxRegion.bind('paste', this.validatePaste);
      }

      var datBoxRegion = this.$el.find('#custom-call-date');
      if (txtBoxRegion.val().length <= 0 || datBoxRegion.val().length <= 0) {
        this.$el.find('#save-custom-call').addClass('btn-disabled');
      } else {
        this.$el.find('#add-custom-call-title').text('Edit Custom Call');
      }
    },
    validateForm: function (e) {
      var $txtBox;
      var $dateBox;
      var $formField = $(e.currentTarget);

      if ($formField.attr('id') === 'custom-call-date') {
        $dateBox = $formField;
        $txtBox = $('#custom-call-data');
      } else if ($formField.attr('id') === 'custom-call-data') {
        $txtBox = $formField;
        var maxLength = $txtBox.attr('maxlength');
        var val = $txtBox.val();
        var valLength = val.length;

        if (valLength > maxLength) {
          $txtBox.val(val.slice(0, maxLength));
        }
        $dateBox = $('#custom-call-date');
      } else {
        $txtBox = $('#custom-call-data');
        $dateBox = $('#custom-call-date');
      }

      var dateBoxValid = App.router.dates.dateFieldIsValid($dateBox);
      if ($txtBox.val().length > 0 && $dateBox.val().length > 0 && dateBoxValid) {
        this.$el.find('#save-custom-call').removeClass('btn-disabled');
      } else {
        this.$el.find('#save-custom-call').addClass('btn-disabled');
      }
    },
    closePopup: function (e) {
      e.preventDefault();
      App.vent.trigger('popup:hideAll');
    },
    saveCall: function () {
      if ($('#save-custom-call').hasClass('btn-disabled')) { return; }
      var txtBox = $('.txt-box');
      var dateBox = $('#custom-call-date');

      var backEndCallDetails = this.model.get('callDetails');

      if (txtBox.length > 0 && txtBox.val().length > 0 && dateBox.length > 0 && dateBox.val().length > 0) {
        var r = txtBox.val();
        var d = App.router.dates.validateDate(dateBox.val());
        d = d.getTime();

        var callDetail = null;
        if (this.callDetail) {       //if callDetail was created before
          callDetail = _.findWhere(backEndCallDetails, {id: this.callDetail.id});
          callDetail.reason = r;
          callDetail.date   = d;
          var cm = new Backbone.Model(callDetail);
          App.vent.trigger('timeline:updateDetailsModel', cm);
        } else {                     //create a new callDetail obj
          callDetail = {
            id: backEndCallDetails.length + 1,
            reason: r,
            date: d,
            callType: 'custom',
            notesEntered: false,
            callsAttempted: false,
            callsCompleted: false
          };
          App.activeCallID = callDetail.id.toString();
          App.vent.trigger('timeline:updateDetailsModel', new Backbone.Model(callDetail));
          App.vent.trigger('updateNextCall', callDetail);

          backEndCallDetails.push(callDetail);
        }
        this.model.save({ callDetails: backEndCallDetails, hasCustomCalls: true },
          {
          success: function () {
            App.vent.trigger('popup:hideAll timeline:update');
          }
        });
      } else {
        App.vent.trigger('popup:hideAll');
      }
    },
    serializeData: function () {
      return {
        getReason: this.callDetailReason,
        getDate: this.callDetailDate
      };
    }
  });
});
