define(function (require) {
  var App = require('app');
  var SearchablePatientView = require('patient-search-searchable');
  var Patients = require('patients');
  var template = require('text!../html/patientSearchListView.html');

  return Backbone.Marionette.CompositeView.extend({
    template : _.template(template),
    itemView: SearchablePatientView,
    itemViewContainer: '#tbody',
    itemViewOptions: function (model, index) {
      return {
        patients: this.patients
      };
    },
    id: 'search-table',
    initialize: function () {
      var that = this;
      var storedPatients = new Patients();
      var restURL = App.Resources.get('patient-list').href + '?facilityId=' + App.Resources.user.vistaLocation;
      storedPatients.fetch({
        url: restURL,
        dataType: 'json',
        reset: true,
        success: function () {
          that.patients = storedPatients;
        }
      });
      that.collection = new Patients();
      this.listenTo(App.vent, 'patients:fetch', this.refresh);
      this.listenTo(App.vent, 'patients:error', this.errorMessage);
    },
    serializeData: function () {
      return {
        resultCount: this.collection.length
      };
    },
    refresh: function (input) {
      this.collection.patientSearchResults(input);
    },
    errorMessage: function (message) {
      this.collection.reset();
      $('.drop-down-text').html('Error: ' + message).focus();
    },
    collectionEvents: {
      'reset': 'render'
    }
  });
});
