define(function (require) {
  var App = require('app');
  var template = require('text!../html/editNotesItemView.html');
  var InfoSubview = require('patient-information-subview');

  return Backbone.Marionette.Layout.extend({
    tagName: 'div',
    id: 'edit-notes-popup',
    template: _.template(template),
    initialize: function () {
      this.subview = new InfoSubview({
        model: this.model
      });
    },
    regions: {
      subviewr: '#pii-region-box'
    },
    events: {
      'click .save-notes-btn': 'saveNotes',
      'click #exit-btn': 'closePopup',
      'keyup #notes-data': 'validateTextBox'
    },
    onRender: function () {
      this.subviewr.show(this.subview);
    },
    validateTextBox: function (e) {
      var $txtBox = $(e.target);
      var maxLength = $txtBox.attr('maxlength');
      var val = $txtBox.val();
      var valLength = val.length;

      if (valLength > maxLength) {
        $txtBox.val(val.slice(0, maxLength));
      }
    },
    closePopup: function (e) {
      e.preventDefault();
      App.vent.trigger('popup:hideAll');

    },
    saveNotes: function () {
      var txt = '';
      var $txtBox = $('.txt-box');
      var maxLength = $txtBox.attr('maxlength');

      if ($txtBox.length > 0) {
        txt = $txtBox.val().trim();
        txt = txt.slice(0, maxLength);
      }
      txt = _.escape(txt);
      this.model.save({notes: txt}, {
        success: function () {
          App.vent.trigger('popup:hideAll notes:render');
        }
      });
    },
    templateHelpers: {
      getNotes: function () {
        return _.unescape(this.notes);
      }
    }

  });
});
