define(function (require) {
  var App = require('app');
  var template = require('text!../html/patientPopup.html');
  var patientId = '';
  var mcc = '';
  var Patient = require('patient-information');
  return Backbone.Marionette.ItemView.extend({
    id: 'patient-dialog',
    tagName: 'div',
    className: '',
    template: _.template(template),

    events: {
      'click #add-mcc-list' : 'skip'
    },
    initialize: function (options) {
      var model = options.model;
      patientId = options.model.get('patientId');
      if (model.get('exists')) {
        this.model.patientFetch(patientId);
        mcc = this.model.get('mcc');
      }
      this.listenTo(App.vent, 'popup:close', this.model.remove);
    },
    onRender: function () {

      if (this.model.get('exists')) {
        this.$el.find('#add-mcc-list').addClass('btn-disabled').html('Patient was found in another MCC List. <br /> Cannot add to this list.');
      } else if (this.model.get('gender') === 'MALE') {
        this.$el.find('#add-mcc-list').addClass('btn-disabled').html('Patient is Male. <br /> Cannot add to list.');
      }
    },
    onShow: function () {

    },
    getPatientDetails: function (e) {
      e.preventDefault();
      if ($('#add-mcc-list').hasClass('btn-disabled')) { return; }
      var model = this.model;
      var url = model.Resources.get('patient-demographics-operational').href;
      $.ajax({url: url, dataType: 'json',
        success: function (response) {
          model.set({
            address: response.address.streetAddressLine1,
            city: response.address.city,
            state: response.address.state,
            zip: response.address.zip,
            email: response.emailAddress
          });
          if (typeof response.phoneNumberMobile !== 'undefined') {
            model.set({phone: response.phoneNumberMobile});
          }
          else if (typeof response.phoneNumberHome !== 'undefined') {
            model.set({
              phone: response.phoneNumberMobile,
              phoneType: 'Home'
            });
          }
          this.savePatient();
        }});

    },
    skip: function (e) {
      e.preventDefault();
      if ($('#add-mcc-list').hasClass('btn-disabled')) { return; }


      this.savePatient();
    },
    savePatient: function () {
      this.model.unset('exists');

      var pii = ['ssn', 'gender', 'dateOfBirth'];
      var attrs = _.omit(this.model.toJSON(), pii);

      var patient = new Patient(attrs);

      App.vent.trigger('popup:hideAll');
      App.router.navigate('edit-patient');
      App.router.controller.showEditPatient(patient);

    },
    templateHelpers: {
      getAge: function (date) {
        return App.router.dates.calculateAge(App.router.dates.validateDate(new Date(date)));
      },
      getDate: function (date) {
        return App.router.dates.convertDate(new Date(date));
      },
      patientExists: function () {
        return this.exists;
      },
      getMcc: function () {
        return mcc;
      }
    }
  });
});
