require_relative 'Functions'
require "json"
require "selenium-webdriver"
gem "test-unit"
require "test/unit"

#gem 'ci_reporter'
#require 'ci/reporter/rake/test_unit_loader'

class HomePage < Test::Unit::TestCase
  def setup
    @driver = Selenium::WebDriver.for :firefox
    @accept_next_alert = true
    @driver.manage.timeouts.implicit_wait = 30
    @verification_errors = []

    set_variables()

  end

  def teardown
    # @driver.save_screenshot('./test/reports/ss_01Navigation.png')
    @driver.quit
    assert_equal [], @verification_errors
  end

  def test_homepage_links
    puts "Running home page link tests"
    #MCC-21 Home page (when NOT logged in)
    accept_eula()
    verify { assert_equal("MCC Support", @driver.title, "Title does not match MCC Support") }
    # header contains the menu dropdown button and the VAHealth logo
    verify { assert(@driver.find_element(:id, "menu-container"), "menu button is not present") }
    verify { assert(@driver.find_element(:id, "logo"), "logo section not present") }
    # footer contains app name/version and logged in status
    verify { assert_equal("MCC Support v1.0.0",@driver.find_element(:class, "version").text) }
    verify { assert(@driver.find_element(:id, "footer").text.include?('Not Logged In'), "Not logged in is not displayed") }
    # verify the name of app, intro text and login button are present in the main page
    verify { assert(@driver.page_source.include?("MCC Support"), "App name not present on home page.") }
    verify { assert(@driver.page_source.include?("Tools and Resources that Support Maternity Care Coordination"), "Info text is not present on home page") }
    verify { assert(@driver.page_source.include?("Tools and Resources that Support Maternity Care Coordination"), "Subfooter text is not present on home page") }
    assert(@driver.find_element(:id, "login-button"), "Login button not present")

    # verify various links in the bottom of the home page when not logged in
    # MCC-551
    @driver.find_element(:link, "About this App").click
    verify { assert(@driver.find_element(:class, "aboutContent").text.include?("This application provides the following resources"), "About text missing") }
    @driver.get(@app_url)
    @driver.find_element(:link, "MCC Patient Call Outlines").click
    verify { assert_equal "Conversation Outlines for Scheduled Calls", @driver.find_element(:id, "call-outlines-heading").text }
    @driver.get(@app_url)
    # outside URLs
    verify { assert_equal("http://www.va.gov/vhapublications/ViewPublication.asp?pub_ID=2803", @driver.find_element(:link, "MCC Handbook 1330.03").attribute("href"), "URL for MCC Handbook was incorrect") }
    verify { assert_equal("http://www.healthquality.va.gov/pregnancy.asp", @driver.find_element(:link, "VA/DoD Guidelines").attribute("href"), "URL for Guidelines was incorrect") }
    # PDF links
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=4", @driver.find_element(:link, "Preventive Health Visit").attribute("href"), "URL for preventative visit was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=6", @driver.find_element(:link, "Post-Partum Follow-Up Visit").attribute("href"), "URL for Postpartum visit was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=11", @driver.find_element(:link, "Patient Education").attribute("href"), "URL for patient education was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=34", @driver.find_element(:link, "Psychosocial Risk Assessment").attribute("href"), "URL for risk assessement was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=15", @driver.find_element(:link, "Prenatal Care Timeline").attribute("href"), "URL for prenatal care was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=27", @driver.find_element(:link, "Resources").attribute("href"), "URL for resources was incorrect") }

    #MCC-21 Home page (when logged in)
    login_to_mcc()
    # title is MCC Support
    verify { assert_equal("MCC Support", @driver.title, "Title does not match MCC Support") }
    # header contains the menu dropdown button and the VAHealth logo
    verify { assert(@driver.find_element(:id, "menu-container"), "menu button is not present") }
    verify { assert(@driver.find_element(:id, "logo"), "logo section not present") }
    # footer contains app name/version and logged in status
    verify { assert_equal(@driver.find_element(:class, "version").text,"MCC Support v1.0.0" ) }
    verify { assert(@driver.find_element(:id, "footer").text.include?(@display_name), "Logged in state not shown") }
    # verify the name of app, intro text and login button are present in the main page
    verify { assert(@driver.page_source.include?("MCC Support"), "App name not present on home page.") }
    verify { assert(@driver.page_source.include?("Tools and Resources that Support Maternity Care Coordination"), "Info text is not present on home page") }

    # verify the 3 links to the site pages are present when logged in
    verify { assert(@driver.find_element(:link, "+Add a Patient\nto the Maternity list"), "Add Patient link no present") }
    verify { assert(@driver.find_element(:link, "View the Maternity\nPatients list"), "View maternity patients link not present") }
    verify { assert(@driver.find_element(:partial_link_text, "See the Call Queue"), "Call list link not present") }


    # verify various links in the bottom of the home page (sub footer)
    # MCC-551
    @driver.find_element(:link, "About this App").click
    verify { assert(@driver.find_element(:class, "aboutContent").text.include?("This application provides the following resources"), "About text missing") }
    @driver.get(@app_url)
    @driver.find_element(:link, "MCC Patient Call Outlines").click
    verify { assert_equal "Conversation Outlines for Scheduled Calls", @driver.find_element(:id, "call-outlines-heading").text }
    @driver.get(@app_url)
    # outside URLs
    verify { assert_equal("http://www.va.gov/vhapublications/ViewPublication.asp?pub_ID=2803", @driver.find_element(:link, "MCC Handbook 1330.03").attribute("href"), "URL for MCC Handbook was incorrect") }
    verify { assert_equal("http://www.healthquality.va.gov/pregnancy.asp", @driver.find_element(:link, "VA/DoD Guidelines").attribute("href"), "URL for Guidelines was incorrect") }
    # PDF links
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=4", @driver.find_element(:link, "Preventive Health Visit").attribute("href"), "URL for preventative visit was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=6", @driver.find_element(:link, "Post-Partum Follow-Up Visit").attribute("href"), "URL for Postpartum visit was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=11", @driver.find_element(:link, "Patient Education").attribute("href"), "URL for patient education was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=34", @driver.find_element(:link, "Psychosocial Risk Assessment").attribute("href"), "URL for risk assessment was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=15", @driver.find_element(:link, "Prenatal Care Timeline").attribute("href"), "URL for prenatal care was incorrect") }
    verify { assert_equal(@app_url + "content/MCCSupport.pdf#page=27", @driver.find_element(:link, "Resources").attribute("href"), "URL for resources was incorrect") }
    logout()
  end

end
