require_relative 'Functions'
require "json"
require "selenium-webdriver"
gem "test-unit"
require "test/unit"

#gem 'ci_reporter'
#require 'ci/reporter/rake/test_unit_loader'

class PatientDetails < Test::Unit::TestCase
  def setup
    @driver = Selenium::WebDriver.for :firefox
    @accept_next_alert = true
    @driver.manage.timeouts.implicit_wait = 30
    @verification_errors = []

    set_variables()

  end

  def teardown
    # @driver.save_screenshot('./test/reports/ss_01Navigation.png')
    @driver.quit
    assert_equal [], @verification_errors
  end

  def test_view_patient_detail
    puts "Running patient detail tests"
    accept_eula()
    login_to_mcc()
    remove_all_patients()

    search_query = "OUTPATIENT, GIRL"
    add_patient(search_query, true)

    @driver.find_element(:class, "menu-toggle").click
    @driver.find_element(:link, "Maternity Patients").click

    #STORY - MCC-33 View Veteran Details

    #MCC-78 Open maternity tracking info
    @driver.find_element(:xpath, "//div[@id='scroll-box']/div/span").click

    wait { @driver.find_element(:id, "patientDetail") }
    wait { @driver.find_element(:class, 'name-line')}
    #MCC-85 View veteran pregnancy information
    verify { assert_equal("OUTPATIENT, GIRL", @driver.find_element(:xpath, "//tr[@id='info-patient-detail']/td").text, "The patient name is not listed correctly.") }
    verify { assert_equal("555-555-5555", @driver.find_element(:xpath, "//div[@id='profile-item-view']/div/div[2]/div[2]/div[2]/strong").text, "The phone number is not listed correctly.") }
    verify { assert_equal("Week", @driver.find_element(:xpath, "//div[@id='profile-item-view']/div/div[2]/div/div").text, "The weeks gestation is not displayed correctly.") }
    verify { assert_equal(@display_name, @driver.find_element(:xpath, "//div[@id='profile-item-view']/div/div[2]/div[4]/div[2]/strong").text, "The MCC Coordinator is not displayed correctly.") }
    verify { assert_equal("Va Doctor", @driver.find_element(:xpath, "//div[@id='profile-item-view']/div/div[2]/div[5]/div[2]/strong").text, "The VA Primary Care Provider is not displayed correctly.") }
    verify { assert_equal("OB Name", @driver.find_element(:xpath, "//div[@id='profile-item-view']/div/div[2]/div[7]/div[2]/strong").text, "The OB provider is not displayed correctly.") }
    verify { assert_equal("555-555-5555", @driver.find_element(:xpath, "//div[@id='profile-item-view']/div/div[2]/div[8]/div[2]/strong").text, "The OB contact information is not displayed correctly.") }
    #MCC-86 Edit patient profile
    verify { assert_equal("Edit Patient Details", @driver.find_element(:id, "edit-patient").text, "Not on the edit patient page.") }
    #MCC-349 comments about the veteran will be displayed in the patient detail screen
    verify { assert_equal("Your notes go here!", @driver.find_element(:id, "notes-body-text").text, "The default notes comment is not viewable.") }

    #STORY - MCC-233 Patient Timeline Details
    #MCC-352 call highlighted will display the date of the call and link to the outline
    # @driver.find_element(:xpath, "//div[@id='timeline']/div[3]/a/div/div").click
    verify { assert(@driver.find_element(:id, "call-detail-date").text) }
    verify { assert(@driver.find_element(:id, "call-detail-type"), "The call outline link is not displayed.") }
    #MCC-351 call queue logic
    verify { assert_equal("May\n24", @driver.find_element(:xpath, "//div[@id='timeline']/div/a/div").text, "The correct date is not displayed.") }
    verify { assert_equal("Jul\n19", @driver.find_element(:xpath, "//div[@id='timeline']/div[2]/a/div").text, "The correct date is not displayed.") }
    verify { assert_equal("Sep\n13", @driver.find_element(:xpath, "//div[@id='timeline']/div[4]/a/div").text, "The correct date is not displayed.") }
    verify { assert_equal("Nov\n8", @driver.find_element(:xpath, "//div[@id='timeline']/div[5]/a/div").text, "The correct date is not displayed.") }
    verify { assert_equal("Dec\n13", @driver.find_element(:xpath, "//div[@id='timeline']/div[6]/a/div").text, "The correct date is not displayed.") }
    verify { assert_equal("Jan\n17", @driver.find_element(:xpath, "//div[@id='timeline']/div[7]/a/div").text, "The correct date is not displayed.") }
    #MCC-83 Call complete indicator
    verify { assert(@driver.find_element(:id, "callsCompleted"), "The Call Complete checkbox is not displayed.") }
    verify { assert(@driver.find_element(:id, "callsAttempted"), "The 3 calls attempted checkbox is not displayed.") }
    verify { assert(@driver.find_element(:id, "notesEntered"), "The notes entered in vista checkbox is not displayed.") }

    #MCC-80 - View next call indicator
    assert_equal("Next Call", @driver.find_element(:id, "call-detail-next").text, "Next Call is not displayed correctly.")

    @driver.find_element(:xpath, "//div[@id='timeline']/div/a/div/div").click
    #STORY - MCC-260 Patient Timeline
    verify { assert_equal("Edit Patient Details", @driver.find_element(:id,"edit-patient").text, "Edit button is missing or mislabeled") }
    #MCC-82 Link to call outline
    verify { assert_equal("Call Outline #2", @driver.find_element(:id,"call-detail-type").text, "Call Outline #2") }


    #STORY - MCC-26 Reminders
    #MCC-62 Call reminder message
    @driver.find_element(:class, "menu-toggle").click
    wait { @driver.find_element(:link, "Home") }
    @driver.find_element(:link, "Home").click

    verify { assert_equal("(4 calls this week)", @driver.find_element(:xpath, "//div[@id='landing']/div/div[2]/a/span/span").text, "The MCC Call reminder is not displayed.") }
    #MCC-63 Navigate to call queue after viewing reminders
    @driver.find_element(:xpath, "//div[@id='landing']/div/div[2]/a").click
    verify {assert_equal("Patient List", @driver.find_element(:css, "span.bold.mylist-title").text, "The call list is not displayed.")}

    #STORY - MCC-108 Delete a Patient
    @driver.find_element(:xpath, "//div[@id='scroll-box']/div/span[4]").click
    @driver.find_element(:id, "edit-patient").click
    verify { assert_equal("Stop MCC Tracking", @driver.find_element(:id, "delete-patient-details").text, "The Stop MCC Tracking link is not displayed.") }
    wait { @driver.find_element(:id, "delete-patient-details") }
    @driver.find_element(:id, "delete-patient-details").click
    verify { assert_equal("Yes, remove the patient from the list.", @driver.find_element(:id, "yes-delete").text, "The confirm button is not displayed.")}
    @driver.find_element(:id, "exit-btn").click
    wait { @driver.find_element(:id, "delete-patient-details") }
    @driver.find_element(:id, "delete-patient-details").click
    @driver.find_element(:id, "no-delete").click
    wait { @driver.find_element(:id, "delete-patient-details") }
    @driver.find_element(:id, "delete-patient-details").click
    @driver.find_element(:id, "yes-delete").click

    remove_all_patients()
    logout()
  end
end

